﻿using System;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Networks.IntegratedOptimization.SurrogateModeling {
  [Item("Point Generator Problem", "")]
  [StorableClass]
  public sealed class PointGeneratorProblem : SingleObjectiveBasicProblem<RealVectorEncoding> {
    public Action<RealVector, double> NotifyEvaluation { get; set; }

    public override bool Maximization {
      get { return false; }
    }

    #region Constructors & Cloning
    [StorableConstructor]
    private PointGeneratorProblem(bool deserializing) : base(deserializing) { }
    private PointGeneratorProblem(PointGeneratorProblem original, Cloner cloner) : base(original, cloner) { }
    public PointGeneratorProblem() { }

    public override IDeepCloneable Clone(Cloner cloner) {

      return new PointGeneratorProblem(this, cloner);
    }
    #endregion

    public override double Evaluate(Individual individual, IRandom random) {
      var point = individual.RealVector();
      var quality = 0.0;

      NotifyEvaluation((RealVector)point.Clone(), quality);

      return quality;
    }

    public override void Analyze(Individual[] individuals, double[] qualities, ResultCollection results, IRandom random) {
      base.Analyze(individuals, qualities, results, random);

      var bestIndividuals = individuals.Zip(qualities, (i, q) => new { Individual = i, Quality = q }).MinItems(x => x.Quality);
      var best = bestIndividuals.First();

      IResult bestSolutionResult;
      if (!results.TryGetValue("BestSolution", out bestSolutionResult)) {
        results.Add(bestSolutionResult = new Result("BestSolution", typeof(RealVector)));
      } else bestSolutionResult = results["BestSolution"];

      IResult bestQualityResult;
      if (!results.TryGetValue("BestQuality", out bestQualityResult)) {
        results.Add(bestQualityResult = new Result("BestQuality", typeof(DoubleValue)));
      } else bestQualityResult = results["BestQuality"];

      var bestQuality = (DoubleValue)bestQualityResult.Value;
      if (bestQuality == null || best.Quality <= bestQuality.Value) {
        bestSolutionResult.Value = (RealVector)best.Individual.RealVector().Clone();
        bestQualityResult.Value = new DoubleValue(best.Quality);
      }
    }
  }
}
