﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Optimization.Operators;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.FacilityLocation {
  [Item("IntegerVector Hamming Similarity Calculator", "Calculates the hamming similarity of two integer vectors (relative amount of identical positions")]
  [StorableClass]
  public class IntegerVectorHammingSimilarityOperator : SingleObjectiveSolutionSimilarityCalculator {

    protected override bool IsCommutative {
      get { return true; }
    }

    [StorableConstructor]
    protected IntegerVectorHammingSimilarityOperator(bool deserializing) : base(deserializing) { }
    protected IntegerVectorHammingSimilarityOperator(IntegerVectorHammingSimilarityOperator original, Cloner cloner) : base(original, cloner) { }
    public IntegerVectorHammingSimilarityOperator() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new IntegerVectorHammingSimilarityOperator(this, cloner);
    }

    public override double CalculateSolutionSimilarity(IScope leftSolution, IScope rightSolution) {
      var a = leftSolution.Variables[SolutionVariableName] as IntegerVector;
      var b = rightSolution.Variables[SolutionVariableName] as IntegerVector;
      if (a == null || b == null) throw new ArgumentException("Scopes do not contain a solution variable.");
      if (a.Length != b.Length) throw new ArgumentException("Solutions are of unequal length.");

      var similarity = 0;
      var len = a.Length;
      for (var i = 0; i < len; i++)
        if (a[i] == b[i]) similarity++;

      return similarity / (double)len;
    }
  }
}
